using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Globalization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security.Permissions;

namespace Prologika.Reporting.Extensibility
{
    public class Util
    {

        private static string XML_SETTING_PATH = "//{0}";
        private static string SETTING_CULTURE_INFO      = "CultureInfo";

        public enum FormatTypeEnum
        {
            None,
            Number,
            Currency,
            Percentage,
            Date
        }
        /// <summary>
        /// Returns a setting value by its name
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <param name="setting">The Setting Name</param>
        /// <returns></returns>
        public static string GetSetting(string settings, string setting)
        {
            if (string.IsNullOrEmpty(settings)) return null;

            XmlNode node;
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(settings);
            
            node = doc.SelectSingleNode(string.Format(XML_SETTING_PATH, setting));
            return node==null?null:node.InnerText;
        }

  
        /// <summary>
        /// Returns a setting value by its name by looking it up in the Settings and ReportPackageSettings
        /// in that order.
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <param name="reportPackageSettings">The ReportPackageSettings XML parameter</param>
        /// <param name="setting">The Setting Name</param>
        /// <returns></returns>
        public static string GetSetting(string settings, string reportPackageSettings, string setting)
        {
            if (string.IsNullOrEmpty(settings) || string.IsNullOrEmpty(reportPackageSettings)) return null;

            string settingValue = GetSetting(settings, setting);
            return settingValue==null?GetSetting(reportPackageSettings, setting):settingValue;
        }

        /// <summary>
        /// Return the format specifier to format general numbers
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <returns></returns>
        public static string GetNumberFormat(string settings)
        {

            string format = null;
            string cultureInfo = GetSetting(settings, SETTING_CULTURE_INFO);

            if (cultureInfo == null) return null;

            CultureInfo ci = DeserializeCulture(cultureInfo);

            // get generic format for positive numbers
            string pf = GetGenericNumberFormatString(ci, true);

            // get generic format for negative numbers
            string nf = GetGenericNumberFormatString(ci, false);

            // construct the generic format specifier for both positive and negative numbers
            format = pf + ";" + nf;

            // get decimal places
            string decimalPlacesExpanded = GetDecimalPlaces(ci, FormatTypeEnum.Number);

            // replace the numeric portion
            format = format.Replace("n", "#" + ci.NumberFormat.NumberGroupSeparator + "##0" + decimalPlacesExpanded);

            return format;
        }
        /// <summary>
        /// Return the format specifier to format dates
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <returns></returns>
        public static string GetPercentFormat(string settings)
        {

            string format = null;
            string cultureInfo = GetSetting(settings, SETTING_CULTURE_INFO);

            if (cultureInfo == null) return null;

            CultureInfo ci = DeserializeCulture(cultureInfo);

            // get generic format for positive numbers
            string pf = GetGenericPercentFormatString(ci, true);

            // get generic format for negative numbers
            string nf = GetGenericPercentFormatString(ci, false);

            // construct the generic format specifier for both positive and negative numbers
            format = pf + ";" + nf;

            // get decimal places
            string decimalPlacesExpanded = GetDecimalPlaces(ci, FormatTypeEnum.Percentage);

            // replace the numeric portion
            format = format.Replace("n", "#" + ci.NumberFormat.PercentGroupSeparator + "##0" + decimalPlacesExpanded);

            return format;
        }

        /// <summary>
        /// Return the format specifier to format dates
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <returns></returns>
        public static string GetDateFormat(string settings)
        {
            //return GetSetting(settings, SETTING_DATE_FORMAT);
            string cultureInfo = GetSetting(settings, SETTING_CULTURE_INFO);

            if (cultureInfo == null) return null;

            CultureInfo ci = DeserializeCulture(cultureInfo);
            return ci.DateTimeFormat.ShortDatePattern;
        }
        /// <summary>
        /// Return the format specifier to format currencies
        /// </summary>
        /// <param name="settings">The Settings XML parameter</param>
        /// <returns></returns>
       public static string GetCurrencyFormat(string settings, bool currencySymbol)
        {

            string format = null;
            string cultureInfo = GetSetting(settings, SETTING_CULTURE_INFO);

            if (cultureInfo == null) return null;

            CultureInfo ci = DeserializeCulture(cultureInfo);
 
            // get generic format for positive numbers
            string pf = GetGenericCurrencyFormatString(ci, true);

            // get generic format for negative numbers
            string nf = GetGenericCurrencyFormatString(ci, false);

            // construct the generic format specifier for both positive and negative numbers
            format = pf + ";" + nf;

            // replace currency symbol
            format = pf.Replace("$", currencySymbol ? ci.NumberFormat.CurrencySymbol : String.Empty);

            // get decimal places
            string decimalPlacesExpanded = GetDecimalPlaces(ci, FormatTypeEnum.Currency);

            // replace the numeric portion
            format = format.Replace("n", "#" + ci.NumberFormat.CurrencyGroupSeparator + "##0" + decimalPlacesExpanded);

            return format;
        }
       /// <summary>
       /// Return the format specifier for a given column in a formula report
       /// </summary>
       /// <param name="paramSettings">The Settings XML-based parameter</param>
       /// <param name="formatInfo">The formatInfo XML fragment</param>
       /// <param name="columnID">The column identifier</param>
       /// <param name="currencySymbol">True, to pre-append the currency symbol for currencies</param>
       /// <returns>The custom format</returns>
       /// <example>=Reporting.Extensibility.Util.GetFormulaFormat(Parameters!Settings.Value, Fields!Settings.Value, "Col_0", False, new Integer(){0, 0, 0, 0})</example>
       public static string GetCustomFormat(string paramSettings, string formatInfo, string columnID, bool currencySymbol)
       {
           // do we have format for this column?
           string setting = GetSetting(formatInfo, columnID);
           if (String.IsNullOrEmpty(setting)) return null;

           // determine the format type
           FormatTypeEnum settingType = (FormatTypeEnum)Enum.Parse(typeof(FormatTypeEnum), setting);

           switch (settingType)
           {
               case FormatTypeEnum.None         : return null;
               case FormatTypeEnum.Number       : return GetNumberFormat(paramSettings);
               case FormatTypeEnum.Currency     : return GetCurrencyFormat(paramSettings, currencySymbol);
               case FormatTypeEnum.Percentage   : return GetPercentFormat(paramSettings);
               case FormatTypeEnum.Date         : return GetDateFormat(paramSettings);
           }

           return null;

       }
       private static string GetGenericNumberFormatString(CultureInfo ci, bool positive)
       {
           string format = null;

           if (positive)
           {
               return "n";
           }
           else
           {
               switch (ci.NumberFormat.NumberNegativePattern)
               {
                   case 0: format = "(n)"; break;
                   case 1: format = "-n"; break;
                   case 2: format = "- n"; break;
                   case 3: format = "n-"; break;
                   case 4: format = "n -"; break;
               }
           }
           return format;
       }

       private static string GetGenericPercentFormatString(CultureInfo ci, bool positive)
       {
           string format = null;

           if (positive)
           {
               switch (ci.NumberFormat.PercentPositivePattern)
               {
                   case 0: format = "n %"; break;
                   case 1: format = "n%"; break;
                   case 2: format = "%n"; break;
                   case 3: format = "% n"; break;
               }
           }
           else
           {
               switch (ci.NumberFormat.NumberNegativePattern)
               {
                   case 0: format = "-n %"; break;
                   case 1: format = "-n%"; break;
                   case 2: format = "-%n"; break;
                   case 3: format = "%-n"; break;
                   case 4: format = "%n-"; break;
                   case 5: format = "n-%"; break;
                   case 6: format = "n%-"; break;
                   case 7: format = "-%n"; break;
                   case 8: format = "n %-"; break;
                   case 9: format = "-% n-"; break;
                   case 10: format = "% -n"; break;
                   case 11: format = "n- %"; break;
               }
           }
           return format;
       }
        private static string GetGenericCurrencyFormatString(CultureInfo ci, bool positive)
        {
            string format = null;

            if (positive)
            {
                switch (ci.NumberFormat.CurrencyPositivePattern)
                {
                    case 0: format = "$n"; break;
                    case 1: format = "n$"; break;
                    case 2: format = "$ n"; break;
                    case 3: format = "n $"; break;
                }
            }
            else
            {
                switch (ci.NumberFormat.CurrencyNegativePattern)
                {
                    case 0: format = "($n)"; break;
                    case 1: format = "-$n"; break;
                    case 2: format = "$-n"; break;
                    case 3: format = "$n-"; break;
                    case 4: format = "(n$)"; break;
                    case 5: format = "-n$"; break;
                    case 6: format = "n-$"; break;
                    case 7: format = "n$-"; break;
                    case 8: format = "-n $"; break;
                    case 9: format = "-$ n"; break;
                    case 10: format = "n $-"; break;
                    case 11: format = "$ n-"; break;
                    case 12: format = "$ -n"; break;
                    case 13: format = "n- $"; break;
                    case 14: format = "($ n)"; break;
                    case 15: format = "(n $)"; break;
                }
            }
            return format;
        }
 
        private static string GetDecimalPlaces(CultureInfo ci, FormatTypeEnum numberFormatType)
        {
            string separator = null;

            switch (numberFormatType)
            {
                case FormatTypeEnum.Currency: separator = ci.NumberFormat.CurrencyDecimalSeparator + String.Empty.PadLeft(ci.NumberFormat.CurrencyDecimalDigits, '0'); break;
                case FormatTypeEnum.Number: separator = ci.NumberFormat.NumberDecimalSeparator + String.Empty.PadLeft(ci.NumberFormat.NumberDecimalDigits, '0'); break;
                case FormatTypeEnum.Percentage: separator = ci.NumberFormat.PercentDecimalSeparator + String.Empty.PadLeft(ci.NumberFormat.PercentDecimalDigits, '0'); break;
            }

            return separator;

        }

        private static CultureInfo DeserializeCulture(string culture)
        {
            try
            {
                SecurityPermission sp = new SecurityPermission(PermissionState.Unrestricted);
                sp.Assert();
            }
            catch (System.Exception ex)
            {
                throw ex;
            }

            CultureInfo ci = (CultureInfo)StringToObject(culture);

            return ci;
        }

        /// <summary>
        /// Serializes an object to Base64String
        /// </summary>
        /// <param name="ObjectToSerialize"></param>
        /// <returns></returns>
        private static string StringFromObject(object ObjectToSerialize)
        {
            MemoryStream ms = new MemoryStream();
            string theSerializedObject;
            BinaryFormatter formatter = new BinaryFormatter();
            try
            {
                formatter.Serialize(ms, ObjectToSerialize);
                byte[] thisByteArray = ms.ToArray();
                theSerializedObject = Convert.ToBase64String(thisByteArray);
            }
            catch (System.Runtime.Serialization.SerializationException ex)
            {
                throw ex;
            }
            finally
            {
                ms.Close();
            }
            return theSerializedObject;
        }


        private static object StringToObject(string StringToDeserialize)
        {
            object thisDeserializedObject = null;
            byte[] thisByteArray = Convert.FromBase64String(StringToDeserialize);
            MemoryStream ms = new MemoryStream(thisByteArray);
            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                thisDeserializedObject = formatter.Deserialize(ms);
            }
            catch (System.Exception ex)
            {

                throw ex;
            }
            finally
            {
                ms.Close();
            }


            return thisDeserializedObject;
        }

        public static object GetValue(string inputValue)
        {
            DateTime dt;
            return DateTime.TryParse(inputValue, out dt)?(object)dt:(object)Decimal.Parse(inputValue); 
        }

        public static string GetQuery(int calendarYear, object[] values, string[] labels)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine(String.Format("DECLARE @CalendarYear int = {0}", calendarYear ));
            sb.AppendLine("DECLARE @Territory TerritoryType");
            

            for (int i = 0; i<values.Length; i++)
            {
                sb.AppendLine(String.Format("insert into @Territory values ({0}, '{1}')", values[i], labels[i]));
            }
            sb.AppendLine("EXECUTE [dbo].[uspGetSalesByTerritory] @CalendarYear,@Territory");
            return sb.ToString();
        }
    }
}
