/*============================================================================
  The Progress Tracker sample was built upon on the Mycostech (www.mycostech.com)
  CylinderTrackBar control sample which can be downloaded from
  http://www.publicjoe.co.uk/csharp/articles/gdn-logo.html
 
  The following portions of the code have been changed/added to fit the CRI needs:
  1. Rotating the control horizontally.
  2. Small rendering changes to reflect the horizontal layout.
  3. Progress value ranges from -1 to 1
  4. Namespace changes
===========================================================================*/
using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Prologika.RS.Extensibility.CRI
{
    public enum WallSide
    {

        /// <summary>
        /// Left
        /// </summary>
        Left,

        /// <summary>
        /// Right
        /// </summary>
        Right,

        /// <summary>
        /// Top
        /// </summary>
        Top,

        /// <summary>
        /// Bottom
        /// </summary>
        Bottom,

        /// <summary>
        /// Back
        /// </summary>
        Back,

        /// <summary>
        /// Front
        /// </summary>
        Front
    }
	/// <summary>
	/// Summary description for DrawingUtils.
	/// </summary>
    public class DrawingUtils
    {
        /// <summary>
        /// Initializes a new instance of the DrawingUtils class.
        /// </summary>
        public DrawingUtils() { }

        /// </summary>
        /// <param name="g">Graphics Object</param>
        /// <param name="front">Rectangle</param>
        /// <param name="fillColor">Fill Color</param>
        /// <param name="borderColor">Border Color</param>
        /// <param name="outLine">Indicated Outline</param>
        public static void DrawBar(Graphics g, Rectangle front, Color fillColor, Color borderColor, bool outLine)
        {
            if (front.Width <= 0 || front.Height <= 0)
                return;

            // Make Back Side Area
            RectangleF aback = front;
            Size depth = new Size(4, 10);

            // Make Depth
            aback.X += depth.Width;

            // Create Top and Bottom Plane.
            RectangleF leftPlane;
            RectangleF rightPlane;

            // Create Graphics Object
            GraphicsPath gp = new GraphicsPath();

            leftPlane = new RectangleF((front.X + aback.X) / 2, aback.Location.Y, aback.X - front.X, front.Height);
            rightPlane = new RectangleF(new PointF((front.X + aback.X) / 2 + front.Width, aback.Location.Y), new SizeF(aback.X - front.X, front.Height));

            // Brush
            SolidBrush brush = new SolidBrush(fillColor);

            // Border Pen
            Pen borderPen = new Pen(borderColor);

            gp.AddEllipse(rightPlane);

            // Get Bottom color
            brush.Color = GetSideColor(fillColor, WallSide.Left);

            // Fill right plane
            g.FillPath(brush, gp);

            // Check Draw Border
            if (outLine) g.DrawPath(borderPen, gp);

            /***************
             *   Left      *
             * ************/
            // Draw left plane
            gp.Reset();

            gp.AddArc(leftPlane, 90, 180);

            gp.AddLine(gp.GetLastPoint(), new PointF((front.X + aback.X) / 2 + front.Width, aback.Top));
            gp.AddArc(rightPlane, -90, 180);
            gp.CloseFigure();

            /***************
             *     Body    *
             * ************/
            // Color For Body is real Fill Color.
            brush.Color = fillColor;

            // Fill Body
            g.FillPath(brush, gp);

            // Shadow of the Body
            FillCylinderShadow(g, front, gp, true);

            // Check Draw Border
            if (outLine) g.DrawPath(borderPen, gp);


            /***************
             *     Right     *
             * ************/
            gp.Reset();
            gp.AddEllipse(rightPlane);

            // Get Bottom color
            brush.Color = GetSideColor(fillColor, WallSide.Right);

            // Fill Top Plane
            g.FillPath(brush, gp);

            // Check Draw Border
            if (outLine) g.DrawPath(borderPen, gp);

            // Dispose
            gp.Dispose();
            brush.Dispose();
            borderPen.Dispose();
        }
        /// <summary>
        /// Draw the Shadow for Cylinder.
        /// </summary>
        /// <param name="g">Graphics Object</param>
        /// <param name="bound">Front Bound</param>
        /// <param name="gp">Graphics Path</param>
        /// <param name="horizontal">Indicated use draw Horionzotal</param>
        public static void FillCylinderShadow(Graphics g, RectangleF bound, GraphicsPath gp, bool horizontal)
        {
            float angle = -180.0f;

            if (horizontal)
                angle = -90.0f;

            LinearGradientBrush linGrBrush = new LinearGradientBrush(
                bound,
                Color.FromArgb(100, 128, 128, 128),     // Opaque black 
                Color.FromArgb(100, Color.White),
                angle);  // Opaque red

            linGrBrush.SetSigmaBellShape(0.70f);

            g.FillPath(linGrBrush, gp);

            linGrBrush.Dispose();
        }//end Method FillCylinderShadow
        /// <summary>
        /// Get Side Color
        /// </summary>
        /// <param name="color"></param>
        /// <param name="side"></param>
        /// <returns>Color</returns>
        public static Color GetSideColor(Color color, WallSide side)
        {
            float bright = 1.0f;

            byte a = color.A;
            byte r = color.R;
            byte g = color.G;
            byte b = color.B;

            switch (side)
            {
                case WallSide.Back:
                    {
                        bright = 1.0f;
                        break;
                    }

                case WallSide.Front:
                    {
                        bright = 1.0f;
                        break;
                    }

                case WallSide.Left:
                    {
                        bright = 0.7f;
                        break;
                    }

                case WallSide.Right:
                    {
                        bright = 0.5f;
                        break;
                    }

                case WallSide.Top:
                    {
                        bright = 0.8f;
                        break;
                    }

                case WallSide.Bottom:
                    {
                        bright = 0.4f;
                        break;
                    }

            }//end swithc case

            r = (byte)(r / 2 + r * bright / 2);
            g = (byte)(g / 2 + g * bright / 2);
            b = (byte)(b / 2 + b * bright / 2);

            return Color.FromArgb(a, r, g, b);
        }
    }

}
