#region Using directives

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;

using Microsoft.ReportDesigner;
using Microsoft.ReportDesigner.Design;
using Microsoft.ReportingServices.Interfaces;
//using RSDrawing = Microsoft.ReportDesigner.Drawing;
using Microsoft.ReportingServices.RdlObjectModel;
#endregion

namespace Prologika.RS.Extensibility.ProgressTrackerCRI
{
    [LocalizedName("Progress Tracker")]
    [ToolboxBitmap(typeof(ProgressTrackerDesigner), "ProgressTracker.ico")]
    [CustomReportItem("ProgressTracker")]
    [Editor(typeof(CustomEditor), typeof(ComponentEditor))]
    public class ProgressTrackerDesigner : CustomReportItemDesigner
    {
        #region Members

        private DesignerVerbCollection _verbs;
        private ProgressTracker _progress;
        private IComponentChangeService _changeService;
        
        #endregion

        #region Constructor and Init

        public ProgressTrackerDesigner()
        {
            _progress = new ProgressTracker();

        }
        /// <summary>
        /// When the item is added on the report canvas,
        /// the Report Designer will invoke this event 
        /// to give a chance to CRI to initialize itself
        /// </summary>
        public override void InitializeNewComponent()
        {
            SetDefaults();
        }
        private void SetDefaults()
        {
            // custom properties
            this.Maximum = 1f;
            this.Minimum = -1f;
            this.Alpha = 255;
            this.Hyperlink = "http://www.prologika.com";
            this.ShowValue = true;
            
            // standard properties
            this.Style.Border.Color = new ReportExpression<ReportColor>(new ReportColor(Color.Black));
            this.Style.Border.Style = new ReportExpression<BorderStyles>(BorderStyles.Solid);
            this.Style.Color = new ReportExpression<ReportColor>(new ReportColor(Color.Gold));
            this.Style.FontSize = new ReportExpression<ReportSize>(new ReportSize(8.0, SizeTypes.Point));
            this.ChangeService().OnComponentChanged(this, null, null, null);
        }
        #endregion

        #region Public Properties

        /// <summary>
        /// Determines the maximum value with the default set to 100
        /// </summary>
        [Category("ProgressTracker"),
        DefaultValue(1f),
        Description("The maximum value.")]
        public float Maximum
        {
            get
            {
                string max = this.GetCustomProperty(Shared.PROP_MAXIMUM);
                return string.IsNullOrEmpty(max) ? 1f : float.Parse(max);
            }
            set
            {
                _progress.Maximum = value;
                SetCustomProperty(Shared.PROP_MAXIMUM, value.ToString());
                Invalidate();
            }
        }

        /// <summary>
        /// Determines the minimum value (defaults to 0)
        /// </summary>
        [Category("ProgressTracker"),
        DefaultValue(-1f),
        Description("The minimum value (negative numbers suppported).")]
        public float Minimum
        {
            get
            {
                string min = this.GetCustomProperty(Shared.PROP_MINIMUM);
                return string.IsNullOrEmpty(min) ? -1f : float.Parse(min);
            }
            set
            {
                _progress.Minimum = value;
                SetCustomProperty(Shared.PROP_MINIMUM, value.ToString());
                Invalidate();
            }
        }

        /// <summary>
        /// Get or set the current Value
        /// </summary>
        [Category("ProgressTracker"), DefaultValue("0"), Description("The actual value.")]
        [Editor(typeof(CustomExpressionEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public string Value
        {
            get
            {
                string v = this.GetCustomProperty(Shared.PROP_VALUE);
                return string.IsNullOrEmpty(v) ? ((_progress.Maximum + _progress.Minimum) / 2).ToString() : v;
            }
            set
            {
                float result;
                if (float.TryParse(value, out result))
                {
                    // numeric value
                    if (result < _progress.Minimum || result > _progress.Maximum)
                        throw new ArgumentException("The progress value must be between the minimum and maximum values.");
                    else
                    {
                        _progress.Value = result;
                    }
                }
                else
                {
                    if (value.Trim() != String.Empty && !value.Trim().StartsWith("="))
                        throw new ArgumentException("Expressions must start with an equal sign.");
                }
                SetCustomProperty(Shared.PROP_VALUE, value); // store in RDL
                this.ChangeService().OnComponentChanged(this, null, null, null);
                Invalidate();
            }
        }

        /// <summary>
        /// Customize the transparency
        /// </summary>
        [Category("ProgressTracker"),
        DefaultValue(255),
        Description("The value of the gradient alpha filter.")]
        public int Alpha
        {
            get
            {
                string alpha = this.GetCustomProperty(Shared.PROP_ALPHA);
                return string.IsNullOrEmpty(alpha) ? 255 : int.Parse(alpha);
            }
            set
            {
                _progress.Alpha = value;
                SetCustomProperty(Shared.PROP_ALPHA, value.ToString());
                Invalidate();
            }
        }

        /// <summary>
        /// Toggle the display of the value when resizing
        /// </summary>
        [Category("ProgressTracker"), DefaultValue(true),
       Description("Set to True to display the value as text inside the control.")]
        public bool ShowValue
        {
            get
            {
                string showValue = this.GetCustomProperty(Shared.PROP_SHOW_VALUE);
                return string.IsNullOrEmpty(showValue) ? true : bool.Parse(showValue);
            }
            set
            {
                _progress.ShowValue = value;
                SetCustomProperty(Shared.PROP_SHOW_VALUE, value.ToString());
                Invalidate();
            }
        }

        /// <summary>
        /// Hyperlink action
        /// </summary>
        [Category("ProgressTracker"), DefaultValue("http://www.prologika.com"), Description("Enter a hyperlink action.")]
        [Editor(typeof(CustomExpressionEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public string Hyperlink
        {
            get
            {
                return this.GetCustomProperty(Shared.PROP_HYPERLINK);
            }
            set
            {
                _progress.Hyperlink = value;
                SetCustomProperty(Shared.PROP_HYPERLINK, value.ToString());
             }
        }

        #endregion Public Properties

        #region Custom Methods

        private string GetCustomProperty(string propertyname)
        {
            if (this.CustomProperties == null)
            {
                return null;
            }

            foreach (KeyValuePair<string, string> property in this.CustomProperties)
            {
                if (property.Key == propertyname)
                    return (string)property.Value;
            }
            return null;
        }

        public void SetCustomProperty(string key, string value)
        {
            CustomProperty customProperty = null;
            foreach (KeyValuePair<string, string> property in this.CustomProperties)
            {
                if (property.Key == key)
                {
                    this.CustomProperties.Remove(property);
                    break;
                }
            }
            customProperty = new CustomProperty();
            this.CustomProperties.Add(new KeyValuePair<string, string>(key,value));

        }

        #endregion

        #region Method overrides

        /// <summary>
        /// CustomReportItemDesigner fires OnPaint each time the CRI needs to be repainted 
        /// </summary>
        public override void Draw(Graphics gr, ReportItemDrawParams dp)
        {
            float value = 0f;
            _progress.Alpha = this.Alpha;
            // sleazy hack to default the progress color to Gold as the standard properties set in InitializeNewComponent
            // don't take effect until the item is rendered in Report Designer
            _progress.ProgressColor = this.Style.Color.Value.Color.Name=="Black" ? Color.Gold : this.Style.Color.Value.Color; 
            _progress.BorderColor = this.Style.Border.Color.Value.Color;
            _progress.Maximum = this.Maximum;
            _progress.Minimum = this.Minimum;
            _progress.Value = float.TryParse(this.Value, out value) ? value : (_progress.Maximum + _progress.Minimum) / 2;
            _progress.ShowValue = this.ShowValue;
            _progress.Font = Shared.GetDrawingFontFromReportFont(this.Style.FontFamily.Value, 
                this.Style.FontStyle.Value.ToString(), this.Style.FontWeight.Value.ToString(), 
                (float) this.Style.FontSize.Value.Value);
            _progress.Height = (int) this.Height;
            _progress.Width = (int) this.Width;
            _progress.DrawControl(gr);
            this.ChangeService();
        }

        public override ItemSize DefaultSize
        {
            get
            {
                return new ItemSize(new Microsoft.ReportingServices.RdlObjectModel.ReportSize(150, SizeTypes.Point),
                    new Microsoft.ReportingServices.RdlObjectModel.ReportSize(30, SizeTypes.Point));
            }
        }

        ///// <summary>
        ///// Fired when the author changes the Font property
        ///// </summary>
        //public override void OnFontChanged()
        //{
        //    base.OnFontChanged();
        //    this.Invalidate();
        //}
        ///// <summary>
        ///// Fired when the author changes the Color property
        ///// </summary>
        //public override void OnColorChanged()
        //{
        //    base.OnColorChanged();
        //    this.Invalidate();
        //}
        public override void OnDragOver(DragEventArgs e)
        {
            IFieldsDataObject fieldsDataObject = e.Data.GetData(typeof(IReportItemDataObject)) as IFieldsDataObject;

            if (fieldsDataObject != null && fieldsDataObject.Fields != null && fieldsDataObject.Fields.Length > 0)
            {
                if (fieldsDataObject.Fields[0].IsNumeric)
                {
                    e.Effect = DragDropEffects.All;
                }
            }
        }
        /// <summary>
        /// Override the DragEnter.  This will enable
        /// the showing of the designer frames by calling the BeginEdit method
        /// <param name="e"></param>
        /// </summary>
        public override void OnDragEnter(DragEventArgs e)
        {
            IFieldsDataObject fieldsDataObject = e.Data.GetData
                (typeof(IReportItemDataObject)) as IFieldsDataObject;

            if (fieldsDataObject != null && fieldsDataObject.Fields != null && 
                fieldsDataObject.Fields.Length > 0)
            {
                BeginEdit();
            }

        }

        public override void OnDragDrop(DragEventArgs e)
        {
            IFieldsDataObject fieldsDataObject = e.Data.GetData(typeof(IReportItemDataObject)) as IFieldsDataObject;

            if (fieldsDataObject != null && fieldsDataObject.Fields != null && fieldsDataObject.Fields.Length > 0)
            {
                if (fieldsDataObject.Fields[0].IsNumeric)
                {
                    this.Value = String.Format("=Fields!{0}.Value", fieldsDataObject.Fields[0].Name);
                }
                else
                    throw new ArgumentException(String.Format("Field '{0}' is not numeric", fieldsDataObject.Fields[0].Name));
            }
        }
        #endregion Method Overrides

        #region Designer Actions

        /// <summary>
        /// Gets the Verbs to implement for the Custom Report Item
        /// </summary>
        public override DesignerVerbCollection Verbs
        {
            get
            {
                if (_verbs == null)
                {
                    _verbs = new DesignerVerbCollection();
                    _verbs.Add(new DesignerVerb("Reset Defaults", new EventHandler(OnCustomAction)));
                }

                return _verbs;
            }
        }

        /// <summary>
        /// Handler for custom report item custom Action
        /// </summary>
        /// <param name="sender">The sender object.</param>
        /// <param name="e">The EventArgs of the event.</param>
        private void OnCustomAction(object sender, EventArgs e)
        {
            switch (((System.ComponentModel.Design.DesignerVerb)sender).Text)
            {
                case "Reset Defaults": SetDefaults(); Invalidate();  break;
                default: MessageBox.Show("Not supported"); break;

            }
        }


        public IComponentChangeService ChangeService()
        {
            if (this._changeService == null)
            {
                this._changeService = (IComponentChangeService)Site.GetService(typeof(IComponentChangeService));
                _changeService.ComponentChanged += new ComponentChangedEventHandler(OnComponentChanged);
            }

            return this._changeService;
        }
        private void OnComponentChanged(object sender, ComponentChangedEventArgs ce)
        {
            Invalidate();
        }
        #endregion

    }



    /// <summary>
    /// Handles the Properties UI
    /// </summary>
    internal sealed class CustomEditor : ComponentEditor
    {
        /// <summary>
        /// Handler for editing properties of the custom report item
        /// </summary>
        /// <param name="context">The context of the component.</param>
        /// <param name="component">The object that is being edited.</param>
        public override bool EditComponent(ITypeDescriptorContext context, object component)
        {
            MessageBox.Show("Implement CRI Properties Window here!");

            return true;
        }

    }
    /// <summary>
    /// Custom Expression Editor
    /// Launches the Reporting Services expression editor to edit an expression-based properties
    /// </summary>
    internal sealed class CustomExpressionEditor : System.Drawing.Design.UITypeEditor
    {
        public override System.Drawing.Design.UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext t)
        {
            return System.Drawing.Design.UITypeEditorEditStyle.Modal;
        }

        public override object EditValue(ITypeDescriptorContext t, System.IServiceProvider s, object oldValue)
        {
            ExpressionEditor editor = new ExpressionEditor();
            object newValue = editor.EditValue(null,
                ((Microsoft.ReportDesigner.CustomReportItemHost)(t.Instance)).CustomReportItemDesigner.Site,
                new ReportExpression(oldValue.ToString()));  
            return newValue != null?newValue:oldValue ;
        }

    } 
}
